# -*- coding: utf-8 -*-
# Author: Manuel de la Pena <manuel@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Controllers with the logic of the UI."""

from PyQt4.QtCore import QUrl

from ubuntu_sso.logger import setup_logging
from ubuntu_sso.utils.ui import (
    CAPTCHA_SOLUTION_ENTRY,
    EMAIL1_ENTRY,
    EMAIL2_ENTRY,
    EXISTING_ACCOUNT_CHOICE_BUTTON,
    FORGOTTEN_PASSWORD_BUTTON,
    JOIN_HEADER_LABEL,
    NAME_ENTRY,
    PASSWORD1_ENTRY,
    PASSWORD2_ENTRY,
    PASSWORD_HELP,
    SET_UP_ACCOUNT_CHOICE_BUTTON,
    SET_UP_ACCOUNT_BUTTON,
    SIGN_IN_BUTTON,
    SURNAME_ENTRY,
    TC_BUTTON,
    VERIFY_EMAIL_TITLE,
    VERIFY_EMAIL_CONTENT,
    YES_TO_TC,
    get_password_strength,
    is_min_required_password)


logger = setup_logging('ubuntu_sso.controllers')


class ChooseSignInController(object):
    """Controlled to the ChooseSignIn view/widget."""

    def __init__(self, title='', existing_account_id=1,
                 new_account_id=2):
        """Create a new instance to manage the view."""
        self._title = title
        self._existing_account_id = existing_account_id
        self._new_account_id = new_account_id

    # use an ugly name just so have a simlar api as found in PyQt
    #pylint: disable=C0103
    def setupUi(self, view):
        """Perform the required actions to set up the ui."""
        view.setTitle(self._title)
        self._set_up_translated_strings(view)
        self._connect_buttons(view)
    #pylint: enable=C0103

    def _set_up_translated_strings(self, view):
        """Set the correct strings for the UI."""
        view.existing_account_button.setText(EXISTING_ACCOUNT_CHOICE_BUTTON)
        view.new_account_button.setText(SET_UP_ACCOUNT_CHOICE_BUTTON)

    def _connect_buttons(self, view):
        """Connect the buttons to the actions to perform."""
        view.existing_account_button.clicked.connect(
                                        lambda: self._set_next_existing(view))
        view.new_account_button.clicked.connect(
                                             lambda: self._set_next_new(view))

    def _set_next_existing(self, view):
        """Set the next id and fire signal."""
        view.next = self._existing_account_id
        view.wizard().next()

    def _set_next_new(self, view):
        """Set the next id and fire signal."""
        view.next = self._new_account_id
        view.wizard().next()


class CurrentUserController(object):
    """Controller used in the view that is used to allow the signin."""

    def __init__(self, title=''):
        """Create a new instance."""
        self._title = title

    def _set_translated_strings(self, view):
        """Set the translated strings."""
        logger.debug('Setting tranlated strings.')
        view.email_edit.setPlaceholderText(EMAIL1_ENTRY)
        view.password_edit.setPlaceholderText(PASSWORD1_ENTRY)
        view.forgot_password_label.setText(FORGOTTEN_PASSWORD_BUTTON)
        view.sign_in_button.setText(SIGN_IN_BUTTON)

    # use an ugly name just so have a simlar api as found in PyQt
    #pylint: disable=C0103
    def setupUi(self, view):
        """Setup the view."""
        view.setTitle(self._title)
        self._set_translated_strings(view)
    #pylint: enable=C0103


class SetUpAccountController(object):
    """Conroller for the setup account view."""

    def __init__(self, tos_id=0, validation_id=1, app_name='',
                 help_message=''):
        """Create a new instance."""
        self._tos_id = tos_id
        self._validation_id = validation_id
        self._app_name = app_name
        self._help_message = help_message

    def _set_translated_strings(self, view):
        """Set the different gettext translated strings."""
        logger.debug('Setting tranlated strings.')
        # set the translated string
        view.name_edit.setPlaceholderText(NAME_ENTRY)
        view.last_name_edit.setPlaceholderText(SURNAME_ENTRY)
        view.email_edit.setPlaceholderText(EMAIL1_ENTRY)
        view.confirm_email_edit.setPlaceholderText(EMAIL2_ENTRY)
        view.password_edit.setPlaceholderText(PASSWORD1_ENTRY)
        view.confirm_password_edit.setPlaceholderText(PASSWORD2_ENTRY)
        view.password_info_label.setText(PASSWORD_HELP)
        view.captcha_solution_edit.setPlaceholderText(CAPTCHA_SOLUTION_ENTRY)
        view.terms_and_conditions_check_box.setText(YES_TO_TC)
        view.terms_and_conditions_button.setText(TC_BUTTON)
        view.set_up_button.setText(SET_UP_ACCOUNT_BUTTON)

    def _set_line_edits_validations(self, view):
        """Set the validations to be performed on the edits."""
        view.set_line_edit_validation_rule(view.email_edit,
                                           self.is_correct_email)
        # set the validation rule for the email confirmation
        view.set_line_edit_validation_rule(view.confirm_email_edit,
                         lambda x: self.is_correct_email_confirmation(x, view))
        # connect the changed text of the password to trigger a changed text
        # in the confirm so that the validation is redone
        view.email_edit.textChanged.connect(
                                      view.confirm_email_edit.textChanged.emit)
        view.set_line_edit_validation_rule(view.password_edit,
                                           is_min_required_password)
        view.set_line_edit_validation_rule(view.confirm_password_edit,
                      lambda x: self.is_correct_password_confirmation(x, view))
        # same as the above case, lets connect a signal to a signal
        view.password_edit.textChanged.connect(
                                   view.confirm_password_edit.textChanged.emit)

    def _connect_ui_elements(self, view):
        """Set the connection of signals."""
        view.terms_and_conditions_button.clicked.connect(
                                               lambda: self.set_next_tos(view))
        view.set_up_button.clicked.connect(lambda: self.set_next_validation(
                                                                         view))
        view.password_edit.textChanged.connect(
                              lambda x: self.update_password_strength(x, view))
        view.terms_and_conditions_check_box.stateChanged.connect(
                                                 view.set_up_button.setEnabled)

    def _set_titles(self, view):
        """Set the diff titles of the view."""
        view.setTitle(JOIN_HEADER_LABEL % {'app_name': self._app_name})
        view.setSubTitle(self._help_message)

    def set_next_tos(self, view):
        """Set the tos page as the next one."""
        view.next = self._tos_id
        view.wizard().next()

    def set_next_validation(self, view):
        """Set the validation as the next page."""
        view.next = self._validation_id
        view.wizard().next()

    def update_password_strength(self, password, view):
        """Callback used to update the password strenght UI code."""
        # get the strengh and then according to it color the frames
        strengh = get_password_strength(password)
        logger.info('Password strengh is %s', strengh)
        view.set_strenght_level(strengh, password)

    def is_correct_email(self, email_address):
        """Return if the email is correct."""
        return '@' in email_address

    def is_correct_email_confirmation(self, email_address, view):
        """Return that the email is the same."""
        return view.email_edit.text() == email_address

    def is_correct_password_confirmation(self, password, view):
        """Return that the passwords are correct."""
        return view.password_edit.text() == password

    #pylint: disable=C0103
    def setupUi(self, view):
        """Setup the view."""
        self._set_titles(view)
        self._set_translated_strings(view)
        self._set_line_edits_validations(view)
        self._connect_ui_elements(view)
    #pylint: enable=C0103


class TosController(object):
    """Controller used for the tos page."""

    def __init__(self, title='', subtitle='', tos_url='http://www.ubuntu.com'):
        """Create a new instance."""
        self._title = title
        self._subtitle = subtitle
        self._tos_url = tos_url

    #pylint: disable=C0103
    def setupUi(self, view):
        """Set up the ui."""
        view.setTitle(self._title)
        view.setSubTitle(self._subtitle)
        # load the tos page
        view.webkit.load(QUrl(self._tos_url))
    #pylint: enable=C0103


class EmailVerificationController(object):
    """Controller used for the verification page."""

    def _set_translated_strings(self, view):
        """Set the trnaslated strings."""
        view.verification_code_edit.setPlaceholderText(VERIFY_EMAIL_TITLE)

    def _connect_ui_elements(self, view):
        """Set the connection of signals."""
        view.next_button.clicked.connect(lambda: self.next_page(view))

    def _set_titles(self, view):
        """Set the different titles."""
        view.setTitle(VERIFY_EMAIL_TITLE)
        view.setSubTitle(VERIFY_EMAIL_CONTENT)

    #pylint: disable=C0103
    def setupUi(self, view):
        """Setup the view."""
        self._set_titles(view)
        self._set_translated_strings(view)
        self._connect_ui_elements(view)
    #pylint: enable=C0103

    def next_page(self, view):
        """Call the next action."""
        view.wizard().next()
